@@ -1,20 +1,195 @@
 ###############################################################################
-# Variable bandwidth selectors
+## Variable bandwidth selectors
 ###############################################################################
 
 
 ###############################################################################
-# Exact MISE for normal mixtures (for pre-clustered KDE)
+## Prepare data x for variable KDE
+##
+## Parameters
+## x - data
+## type - "abram" - for Abramson
+##      - "sain" - for Sain (2002)
+##      - "clust" - pre-clustered
+##
+## Returns
+## list of with fields
+## x - data
+## nclust - number of data points in each cluster 
+## in - index of cluster membership
+###############################################################################
+
+
+pre.vkde <- function(x, type, G, pre="scale", small=5, nu, dist="euclidean",
+    link="average", stop.rule="dudahart", sig.level=0.001)
+{
+  typ <- substr(tolower(type),1,1)
+
+  if (typ=="a")
+    x.pvk <- pre.abram(x=x)
+  else if (typ=="s")
+    x.pvk <- pre.sain(x=x, G=G, pre=pre, small=small)
+  else if (typ=="c")
+    x.pvk <- pre.clust(x=x, nu=nu, dist=dist, link=link, stop.rule=stop.rule,
+                      sig.level=sig.level, pre=pre)
+
+  class(x.pvk) <- "pvkde"
+  return (x.pvk)
+}
+
+
+###############################################################################
+## Plot pre-clustered data 
 #
-# Parameters
-# mus - means
-# Sigmas - variances
-# Props - vector of proportions of each mixture component 
-# Hpc - bandwidth matrices
-# samppc - sample sizes
+## Parameters
+## x.pc - pre-clustered data points
 #
-# Returns
-# Exact MISE for normal mixtures
+## Returns
+## Plots data with each cluster denoted 1,2,... and in col=1,2,...
+###############################################################################
+
+plot.pvkde <- function(x, ...)
+{
+  x.pvk <- x
+  rm(x)
+  plot(x.pvk$x, cex=0, ...)
+  text(x.pvk$x[,1], x.vk$x[,2], x.pvk$ind, col=x.pvk$ind)
+}
+
+
+
+Hlscv.vkde <- function(x, Hstart, pre="scale", univar=FALSE)
+{
+  if (length(x$nclust)==length(x$x))
+    H <- Hlscv.ab(x, pre=pre)
+  else
+    H <- Hlscv.pc(x, Hstart=Hstart, univar=univar)
+}
+
+
+
+###############################################################################
+### Abramsom selectors 
+###############################################################################
+
+###############################################################################
+## Prepare data x for Abramson variable KDE 
+##############################################################################
+
+pre.abram <- function(x)
+{
+  if (is.vector(x)) n <- 1
+  else n <- nrow(x)
+  return(pre.clust(x, nu=n))
+}
+
+
+###############################################################################
+## LSCV for Abramson type bandwidth matrices using normal reference pilot KDE
+## Parameters
+## x - data values
+#
+## Returns
+## H_LSCV,AB
+###############################################################################
+
+lscv.mat.ab <- function(x, h, fhatp)
+{
+  d <- ncol(x)
+  n <- nrow(x)
+  h.ab <- h^2 * fhatp^(-1) 
+
+  if (is.vector(x))
+  {
+    n <- 1; d <- 1; x1 <- x[1]; x2 <- x[2]
+  }
+  else
+  {
+    n <- nrow(x); x1 <- x[,1]; x2 <- x[,2]
+  }
+  result <- .C("dmvnorm_2d_sum_ab", as.double(x1), as.double(x2), as.double(h.ab),
+               as.double(h.ab), as.integer(n), as.integer(1), as.double(0),
+               PACKAGE="ks")
+  lscv1 <- result[[7]]
+
+  result <- .C("dmvnorm_2d_sum_ab", as.double(x1), as.double(x2), as.double(h.ab),
+               as.double(h.ab), as.integer(n), as.integer(0), as.double(0),
+               PACKAGE="ks")
+  lscv2 <- result[[7]]
+  
+  return(lscv1/n^2 - 2/(n*(n-1))*lscv2)
+}
+
+###############################################################################
+## Find Abramson type bandwidth matrices with LSCV 
+## 
+## Parameters
+## x - data values
+## pre - "scale" - pre-scaled data
+##     - "sphere"- pre-sphered data
+#
+## Returns
+## H_LSCV,A
+###############################################################################
+
+Hlscv.ab <- function(x, pre="scale")
+{
+  d <- 2
+  if (substr(pre,1,2)=="sc")
+    pre <- "scale"
+  else if (substr(pre,1,2)=="sp")
+    pre <- "sphere"
+  if(!is.matrix(x)) x <- as.matrix(x)
+
+  ## pre-transform data
+  if (pre=="sphere")
+    x.star <- pre.sphere(x)
+  else if (pre=="scale")
+    x.star <- pre.scale(x)
+  S.star <- var(x.star)
+  n <- nrow(x.star)
+
+  G.nr.star <- (4/(n*(d+2)))^(2/(d+4)) * S.star
+  fhatp <- kde(x.star, G.nr.star, eval.points=x.star)$est
+
+  lscv.mat.ab.temp <- function(h)
+  {
+    return(lscv.mat.ab(x.star, h, fhatp))
+  }
+
+  hend <- sum(diag((4/(n*(d+2)))^(2/(d+4))*var(x)*5))
+  result <- optimise(lscv.mat.ab.temp, interval=c(0, hend))
+  H.ab <- result$min^2 * diag(d)
+
+  if (pre=="scale") S12 <- diag(sqrt(diag(var(x))))
+  else if (pre=="sphere") S12 <- matrix.sqrt(var(x))
+  H.ab <- S12 %*% H.ab %*% S12
+
+  H.lscv.ab <- numeric()
+  for (i in 1:nrow(x))
+    H.lscv.ab <- rbind(H.lscv.ab, H.ab * fhatp[i]^(-1))
+  
+  return(H.lscv.ab)
+}
+
+
+###############################################################################
+### Pre-clustered selectors
+###############################################################################
+
+
+###############################################################################
+## Exact MISE for normal mixtures (for pre-clustered KDE)
+#
+## Parameters
+## mus - means
+## Sigmas - variances
+## Props - vector of proportions of each mixture component 
+## Hpc - bandwidth matrices
+## samppc - sample sizes
+#
+## Returns
+## Exact MISE for normal mixtures
 ###############################################################################
 
 mise.mixt.pc <- function(H.pc, mus, Sigmas, props, samp.pc)
@@ -56,17 +231,17 @@ mise.mixt.pc <- function(H.pc, mus, Sigmas, props, samp.pc)
 }
 
 ###############################################################################
-# ISE for normal mixtures (pre-clustered data)
-# 
-# Parameters
-# x.pc - pre-clustered data values
-# H.pc - bandwidth matrices
-# mus - matrix of means
-# Sigmas - matrix of covariance matrices 
-# props - mixing proportions 
+## ISE for normal mixtures (pre-clustered data)
+## 
+## Parameters
+## x.pc - pre-clustered data values
+## H.pc - bandwidth matrices
+## mus - matrix of means
+## Sigmas - matrix of covariance matrices 
+## props - mixing proportions 
 #
-# Returns
-# ISE
+## Returns
+## ISE
 ###############################################################################
 
 ise.mixt.pc <- function(x.pc, H.pc, mus, Sigmas, props, lower, upper,
@@ -95,30 +270,30 @@ ise.mixt.pc <- function(x.pc, H.pc, mus, Sigmas, props, lower, upper,
   
   #for (i in 1:n)
   #{
-  #  clusti <- which(x.pc$ind[i]==ind.lab)
-  #  Hi <- H.pc[((clusti-1)*d+1):(clusti*d),]
-  #  
-  #  for (ip in 1:n)
-  #  {
-  #    clustip <- which(x.pc$ind[i]==ind.lab)
-  #    Hip <- H.pc[((clustip-1)*d+1):(clustip*d),]
-  #    ise1 <- ise1 + dmvnorm.mixt(x[i,], mus=x[ip,], Sigmas=Hi + Hip, props=1)
-  #  }
+  ##  clusti <- which(x.pc$ind[i]==ind.lab)
+  ##  Hi <- H.pc[((clusti-1)*d+1):(clusti*d),]
+  ##  
+  ##  for (ip in 1:n)
+  ##  {
+  ##    clustip <- which(x.pc$ind[i]==ind.lab)
+  ##    Hip <- H.pc[((clustip-1)*d+1):(clustip*d),]
+  ##    ise1 <- ise1 + dmvnorm.mixt(x[i,], mus=x[ip,], Sigmas=Hi + Hip, props=1)
+  ##  }
    
-  #  for (k in 1:m)
-  #  {
-  #    Sigmak <- Sigmas[((k-1)*d+1):(k*d),]
-  #    ise2 <- ise2 + props[k]*dmvnorm.mixt(x[i,], mus=mus[k,], Sigmas=Hi + Sigmak, props=1)
-  #  }
+  ##  for (k in 1:m)
+  ##  {
+  ##    Sigmak <- Sigmas[((k-1)*d+1):(k*d),]
+  ##    ise2 <- ise2 + props[k]*dmvnorm.mixt(x[i,], mus=mus[k,], Sigmas=Hi + Sigmak, props=1)
+  ##  }
   #}
   #for (k in 1:m)
   #{
-  #  Sigmak <- Sigmas[((k-1)*d+1):(k*d),]
-  #  for (kp in 1:m)
-  #  {
-  #    Sigmakp <- Sigmas[((kp-1)*d+1):(kp*d),]
-  #    ise3 <- ise3 + props[k]*props[kp]*dmvnorm.mixt(mus[k,], mus[kp,], Sigmakp + Sigmak, props=1)
-  #  }
+  ##  Sigmak <- Sigmas[((k-1)*d+1):(k*d),]
+  ##  for (kp in 1:m)
+  ##  {
+  ##    Sigmakp <- Sigmas[((kp-1)*d+1):(kp*d),]
+  ##    ise3 <- ise3 + props[k]*props[kp]*dmvnorm.mixt(mus[k,], mus[kp,], Sigmakp + Sigmak, props=1)
+  ##  }
   #}
   
   #ise <- ise1/n^2 - 2*ise2/n + ise3
@@ -146,183 +321,21 @@ ise.mixt.pc <- function(x.pc, H.pc, mus, Sigmas, props, lower, upper,
 
 
 
+
 ###############################################################################
-# Prepare data x for variable KDE
+## Finds the local mimina in matrix of data values for 2D
 #
-# Parameters
-# x - data
-# type - "abram" - for Abramson
-#      - "sain" - for Sain (2002)
-#      - "clust" - pre-clustered
+## Parameters
+## heights - data matrix
 #
-# Returns
-# list of with fields
-# x - data
-# nclust - number of data points in each cluster 
-# in - index of cluster membership
-###############################################################################
-
-
-pre.vkde <- function(x, type, G, pre="scale", small=5, nu, dist="euclidean",
-    link="average", stop.rule="dudahart", sig.level=0.001)
-{
-  typ <- substr(tolower(type),1,1)
-
-  if (typ=="a")
-    x.pvk <- pre.abram(x=x)
-  else if (typ=="s")
-    x.pvk <- pre.sain(x=x, G=G, pre=pre, small=small)
-  else if (typ=="c")
-    x.pvk <- pre.clust(x=x, nu=nu, dist=dist, link=link, stop.rule=stop.rule,
-                      sig.level=sig.level, pre=pre)
-
-  class(x.pvk) <- "pvkde"
-  return (x.pvk)
-}
-
-
-###############################################################################
-# Plot pre-clustered data 
-#
-# Parameters
-# x.pc - pre-clustered data points
-#
-# Returns
-# Plots data with each cluster denoted 1,2,... and in col=1,2,...
-###############################################################################
-
-plot.pvkde <- function(x, ...)
-{
-  x.pvk < x
-  rm(x)
-  plot(x.pvk$x, cex=0, ...)
-  text(x.pvk$x[,1], x.vk$x[,2], x.pvk$ind, col=x.pvk$ind)
-}
-
-Hlscv.vkde <- function(x, Hstart, pre="scale", univar=FALSE)
-{
-  if (length(x$nclust)==length(x$x))
-    H <- Hlscv.ab(x, pre=pre)
-  else
-    H <- Hlscv.pc(x, Hstart=Hstart, univar=univar)
-}
-
-
-###############################################################################
-# Prepare data x for Abramson variable KDE 
-##############################################################################
-
-pre.abram <- function(x)
-{
-  if (is.vector(x)) n <- 1
-  else n <- nrow(x)
-  return(pre.clust(x, nu=n))
-}
-
-
-###############################################################################
-# LSCV for Abramson type bandwidth matrices using normal reference pilot KDE
-# Parameters
-# x - data values
-#
-# Returns
-# H_LSCV,AB
-###############################################################################
-
-lscv.mat.ab <- function(x, h, fhatp)
-{
-  d <- ncol(x)
-  n <- nrow(x)
-  h.ab <- h^2 * fhatp^(-1) 
-
-  if (is.vector(x))
-  {
-    n <- 1; d <- 1; x1 <- x[1]; x2 <- x[2]
-  }
-  else
-  {
-    n <- nrow(x); x1 <- x[,1]; x2 <- x[,2]
-  }
-  result <- .C("dmvnorm_2d_sum_ab", as.double(x1), as.double(x2), as.double(h.ab),
-               as.double(h.ab), as.integer(n), as.integer(1), as.double(0),
-               PACKAGE="ks")
-  lscv1 <- result[[7]]
-
-  result <- .C("dmvnorm_2d_sum_ab", as.double(x1), as.double(x2), as.double(h.ab),
-               as.double(h.ab), as.integer(n), as.integer(0), as.double(0),
-               PACKAGE="ks")
-  lscv2 <- result[[7]]
-  
-  return(lscv1/n^2 - 2/(n*(n-1))*lscv2)
-}
-
-###############################################################################
-# Find Abramson type bandwidth matrices with LSCV 
-# 
-# Parameters
-# x - data values
-# pre - "scale" - pre-scaled data
-#     - "sphere"- pre-sphered data
-#
-# Returns
-# H_LSCV,A
-###############################################################################
-
-Hlscv.ab <- function(x, pre="scale")
-{
-  d <- 2
-  if (substr(pre,1,2)=="sc")
-    pre <- "scale"
-  else if (substr(pre,1,2)=="sp")
-    pre <- "sphere"
-  if(!is.matrix(x)) x <- as.matrix(x)
-
-  # pre-transform data
-  if (pre=="sphere")
-    x.star <- pre.sphere(x)
-  else if (pre=="scale")
-    x.star <- pre.scale(x)
-  S.star <- var(x.star)
-  n <- nrow(x.star)
-
-  G.nr.star <- (4/(n*(d+2)))^(2/(d+4)) * S.star
-  fhatp <- kde(x.star, G.nr.star, eval.points=x.star)$est
-
-  lscv.mat.ab.temp <- function(h)
-  {
-    return(lscv.mat.ab(x.star, h, fhatp))
-  }
-
-  hend <- sum(diag((4/(n*(d+2)))^(2/(d+4))*var(x)*5))
-  result <- optimise(lscv.mat.ab.temp, interval=c(0, hend))
-  H.ab <- result$min^2 * diag(d)
-
-  if (pre=="scale") S12 <- diag(sqrt(diag(var(x))))
-  else if (pre=="sphere") S12 <- matrix.sqrt(var(x))
-  H.ab <- S12 %*% H.ab %*% S12
-
-  H.lscv.ab <- numeric()
-  for (i in 1:nrow(x))
-    H.lscv.ab <- rbind(H.lscv.ab, H.ab * fhatp[i]^(-1))
-  
-  return(H.lscv.ab)
-}
-
-
-###############################################################################
-# Finds the local mimina in matrix of data values for 2D
-#
-# Parameters
-# heights - data matrix
-#
-# Returns
-# matrix of indices of local minima
+## Returns
+## matrix of indices of local minima
 ###############################################################################
 
 
 mvpeaks.2d <- function(height, span=3)
 {
-  # function by Petr Pikal 
+  ## function by Petr Pikal 
   peaks <- function (series, span = 3)
   {
     z <- embed(series, span)
@@ -336,7 +349,7 @@ mvpeaks.2d <- function(height, span=3)
   nc <- ncol(height)
   nr <- nrow(height)
 
-  # find modes in x and y directions
+  ## find modes in x and y directions
   x.max <- vector()
   y.max <- vector()
   for (i in 2:(nc-1))  
@@ -345,7 +358,7 @@ mvpeaks.2d <- function(height, span=3)
   for (i in 2:(nr-1))
     y.max <- rbind(y.max, peaks(height[i,], span=span))
 
-  # combine modes for each direction
+  ## combine modes for each direction
   modes.ind <- which(x.max & y.max, arr.ind=TRUE)
 
   return(modes.ind)
@@ -353,14 +366,14 @@ mvpeaks.2d <- function(height, span=3)
 
 
 ###############################################################################
-# Finds the nearest mode to data point
+## Finds the nearest mode to data point
 #
-# Parameters
-# x - data matrix
-# modes - matrix of sample modes
+## Parameters
+## x - data matrix
+## modes - matrix of sample modes
 #
-# Returns
-# vector of indices of nearest mode 
+## Returns
+## vector of indices of nearest mode 
 ###############################################################################
 
 nearest.mode <- function(x, modes)
@@ -377,19 +390,19 @@ nearest.mode <- function(x, modes)
 }
 
 ###############################################################################
-# Prepare data points for Sain's partitioned b/w selector  
+## Prepare data points for Sain's partitioned b/w selector  
 #
-# Parameters
-# x - data matrix
-# G - pilot b/w matrix
-# pre - pre-transformation
-# small - smallest number of points allowed to form a separate cluster
+## Parameters
+## x - data matrix
+## G - pilot b/w matrix
+## pre - pre-transformation
+## small - smallest number of points allowed to form a separate cluster
 #
-# Returns
-# list of values with components (suitable for use with Hlscv.pc)
-# x - data values
-# ind - vector of indices indicating which partition class
-# nclust - vector of number of points in each partition class 
+## Returns
+## list of values with components (suitable for use with Hlscv.pc)
+## x - data values
+## ind - vector of indices indicating which partition class
+## nclust - vector of number of points in each partition class 
 ###############################################################################
 
 pre.sain <- function(x, G, pre="scale", small=5)
@@ -401,7 +414,7 @@ pre.sain <- function(x, G, pre="scale", small=5)
     pre <- "sphere"
   if(!is.matrix(x)) x <- as.matrix(x)
 
-  # pre-transform data
+  ## pre-transform data
   if (pre=="sphere")
     x.star <- pre.sphere(x)
   else if (pre=="scale")
@@ -409,14 +422,14 @@ pre.sain <- function(x, G, pre="scale", small=5)
   S.star <- var(x.star)
   n <- nrow(x.star)
 
-  # pilot KDE
+  ## pilot KDE
   if (missing(G)) G <- (4/(n*(d+2)))^(2/(d+4)) * S.star
   fhatp <- kde(x.star, G, gridsize=c(100,100))
 
-  # find indices for sample modes for pilot KDE
+  ## find indices for sample modes for pilot KDE
   modes.ind <- mvpeaks.2d(fhatp$estimate)
  
-  # find value of modes from indices
+  ## find value of modes from indices
   if (is.vector(modes.ind))
     modes <- c(fhatp$eval.points[[1]][modes.ind[1]],
                fhatp$eval.points[[2]][modes.ind[2]])
@@ -428,7 +441,7 @@ pre.sain <- function(x, G, pre="scale", small=5)
                              fhatp$eval.points[[2]][modes.ind[i,2]]))
   }
 
-  # associate data points to nearest mode
+  ## associate data points to nearest mode
   if (is.vector(modes))
   { 
     modes.lab <- rep(1, length=nrow(x))
@@ -474,12 +487,12 @@ pre.sain <- function(x, G, pre="scale", small=5)
   }
   #if (plot.it)
   #{
-  #  plot.kde(fhatp, cont=seq(0,100, by=10), col="grey", cex=0)
-  #  text(x.star[,1], x.star[,2], modes.lab)
-  #  if (is.vector(modes))
-  #    text(modes[1], modes[2], col=2)
-  #  else
-  #    text(modes, col=2)
+  ##  plot.kde(fhatp, cont=seq(0,100, by=10), col="grey", cex=0)
+  ##  text(x.star[,1], x.star[,2], modes.lab)
+  ##  if (is.vector(modes))
+  ##    text(modes[1], modes[2], col=2)
+  ##  else
+  ##    text(modes, col=2)
   #}
   
   return(list(x=x, ind=modes.lab, nclust=nclust))
@@ -487,21 +500,21 @@ pre.sain <- function(x, G, pre="scale", small=5)
 
 
 ###############################################################################
-# Pre-cluster data set into nu clusters
+## Pre-cluster data set into nu clusters
 #
-# Parameters
-# x - data points
-# nu - number of clusters
-# dist - distance function
-# link - linking function
-# stop.rule  - stopping rule
-# sig.level - (step-wise only) significance level
+## Parameters
+## x - data points
+## nu - number of clusters
+## dist - distance function
+## link - linking function
+## stop.rule  - stopping rule
+## sig.level - (step-wise only) significance level
 #
-# Returns
-# List with elements
-# x - data matrix
-# ind - index labels to indicate cluster membership
-# nclust - number of members in each cluster
+## Returns
+## List with elements
+## x - data matrix
+## ind - index labels to indicate cluster membership
+## nclust - number of members in each cluster
 ###############################################################################
 
 pre.clust <- function(x, nu, dist="euclidean", link="average",
@@ -509,7 +522,7 @@ pre.clust <- function(x, nu, dist="euclidean", link="average",
 {
   n <- nrow(x)
   
-  # pre-transform data before clustering
+  ## pre-transform data before clustering
   if (pre=="sphere")
     xstar <- pre.sphere(x)
   else
@@ -518,7 +531,7 @@ pre.clust <- function(x, nu, dist="euclidean", link="average",
   xd <- dist(xstar, method=dist)
   dendo <- hclust(xd, method=link)
 
-  # find optimal number of clusters according to stopping rule
+  ## find optimal number of clusters according to stopping rule
   if (missing(nu))
     nu <- optnum.clust(xstar, dendo, stop.rule=stop.rule, sig.level=sig.level)
   clust.ind <- cutree(dendo, k=nu)
@@ -533,15 +546,15 @@ pre.clust <- function(x, nu, dist="euclidean", link="average",
 
 
 ###############################################################################
-# Sum of squares decomposition
+## Sum of squares decomposition
 #
-# Parameters
-# x.pc - data matrix
+## Parameters
+## x.pc - data matrix
 #
-# Returns
-# List with elements
-# B - between clusters sum of squares
-# W - within clusters sum of squares
+## Returns
+## List with elements
+## B - between clusters sum of squares
+## W - within clusters sum of squares
 ###############################################################################
 
 ssq.decomp <- function(x.pc)
@@ -565,8 +578,8 @@ ssq.decomp <- function(x.pc)
     return(ssq)
   }
   
-  xW <- numeric()   # xW is x (in cluster) - cluster mean 
-  xB <- numeric()   # XB is x (in cluster) - total mean
+  xW <- numeric()   ## xW is x (in cluster) - cluster mean 
+  xB <- numeric()   ## XB is x (in cluster) - total mean
   count <- 0
 
   for (i in clust.indg)
@@ -574,7 +587,7 @@ ssq.decomp <- function(x.pc)
     count <- count + 1
     xpci <- x[clust.ind==i,]
 
-    # more than one element in cluster
+    ## more than one element in cluster
     if (is.matrix(xpci))
     {
       xbari <- apply(xpci, 2, mean)
@@ -582,7 +595,7 @@ ssq.decomp <- function(x.pc)
       for (j in 1:nclust[count])
         xW <- rbind(xW, xpci[j,] - xbari)
     }
-    # only one element in cluster
+    ## only one element in cluster
     else
     {
       xW <- rbind(xW, rep(0, d))
@@ -597,16 +610,16 @@ ssq.decomp <- function(x.pc)
  
 
 ###############################################################################
-# Find optimal number of clusters in data set
+## Find optimal number of clusters in data set
 #
-# Parameters
-# x - data points
-# dendo - dendogram (output from hclust)
-# stop.rule - stopping rule
-# sig.level - (step-wise only) significance level of test
+## Parameters
+## x - data points
+## dendo - dendogram (output from hclust)
+## stop.rule - stopping rule
+## sig.level - (step-wise only) significance level of test
 #
-# Returns
-# Optimal number of clusters
+## Returns
+## Optimal number of clusters
 ###############################################################################
 
 optnum.clust <- function(x, dendo, stop.rule, sig.level)
@@ -615,11 +628,11 @@ optnum.clust <- function(x, dendo, stop.rule, sig.level)
   n <- nrow(x)
   tree.temp <- clust.tree(dendo)
   tree <- tree.temp$tree
-  # tree contains all info needed to reconstruct hierarchy
+  ## tree contains all info needed to reconstruct hierarchy
   tree.ind <- tree.temp$ind
-  nu <- 1 # nu is number of clusters 
+  nu <- 1 ## nu is number of clusters 
 
-  # initial test - compare SS for 1 cluster with that for 2 clusters
+  ## initial test - compare SS for 1 cluster with that for 2 clusters
   i <-  nrow(tree)
   W1 <- var(x) * (n-1)
   tree.ind2 <- sort(tree[i,1:2])
@@ -640,16 +653,16 @@ optnum.clust <- function(x, dendo, stop.rule, sig.level)
  
     while ((teststat < critval) & (nu < n))
     {
-      i <- i - 1   # move backwards up tree
-      nu <- nu + 1 # increment number of clusters 
+      i <- i - 1   ## move backwards up tree
+      nu <- nu + 1 ## increment number of clusters 
       ind1 <- tree[i,3]
       tree.ind2 <- sort(tree[i,1:2])
-      nc1 <- length(which(tree.ind[,i+1]==ind1)) # number in cluster 1
-      wti1 <- which(tree.ind[,i]==tree.ind2[1])  # indices of elem in clust 1 
-      wti2 <- which(tree.ind[,i]==tree.ind2[2])  # indices of elem in clust 2
+      nc1 <- length(which(tree.ind[,i+1]==ind1)) ## number in cluster 1
+      wti1 <- which(tree.ind[,i]==tree.ind2[1])  ## indices of elem in clust 1 
+      wti2 <- which(tree.ind[,i]==tree.ind2[2])  ## indices of elem in clust 2
       x.ind1 <- tree.ind[,i+1]==ind1
 
-      # reconstruct cluster 1 x.pc1 and cluster 2 x.pc2 
+      ## reconstruct cluster 1 x.pc1 and cluster 2 x.pc2 
       x.pc1 <- list(x=x[which(x.ind1),], ind=tree.ind[x.ind1,i+1], nclust=nc1)
       x.ind2 <- (tree.ind[,i] ==tree.ind2[1]) | (tree.ind[,i]==tree.ind2[2])
       nc2 <- c(length(wti1), length(wti2))
@@ -669,29 +682,29 @@ optnum.clust <- function(x, dendo, stop.rule, sig.level)
 }
 
 ###############################################################################
-# Reconstructs clustering hierarchy 
+## Reconstructs clustering hierarchy 
 #
-# Parameters
-# dendo - output from hclust (of n data points)
+## Parameters
+## dendo - output from hclust (of n data points)
 #
-# Returns
+## Returns
 #
-# (n-1) x 3 matrix with i-th row: 
-# Columns 1, 2 - index label (output from cutree(dendo, k=i)) of the two
-#                clusters being merged
-# Column 3 - index label of merged cluster (output from cutree(dendo, k=i+1))
+## (n-1) x 3 matrix with i-th row: 
+## Columns 1, 2 - index label (output from cutree(dendo, k=i)) of the two
+##                clusters being merged
+## Column 3 - index label of merged cluster (output from cutree(dendo, k=i+1))
 ###############################################################################
 
 clust.tree <- function(dendo)
 { 
   merge <- dendo$merge
-  # merge is what we need to reconstruct hierarchy
+  ## merge is what we need to reconstruct hierarchy
   n <- nrow(merge) + 1
   clust.ind <- as.matrix(cutree(dendo, k=n:1))
   clust.indn <- clust.ind[,-n]
   index <- matrix(0, nrow=n-1, ncol=3)
 
-  # modify 'merge' to format which 'optnum.clust' requires as input 
+  ## modify 'merge' to format which 'optnum.clust' requires as input 
   for (i in 1:(n-1))
   {
     m <- merge[i,]
@@ -713,14 +726,14 @@ clust.tree <- function(dendo)
 
 
 ###############################################################################
-# Pre-clustered bandwidth selectors
+## Pre-clustered bandwidth selectors
 #
-# Parameters
-# x.pc - pre-clustered data points
-# H.pc - bandwidth matrices
+## Parameters
+## x.pc - pre-clustered data points
+## H.pc - bandwidth matrices
 #
-# Returns
-# Bandwidth matrix for each cluster
+## Returns
+## Bandwidth matrix for each cluster
 ###############################################################################
 
 lscv.mat.pc <- function(x.pc, H.pc)
@@ -760,14 +773,14 @@ lscv.mat.pc <- function(x.pc, H.pc)
 
 
 ###############################################################################
-# Finds the bandwidth matrices that minimises LSCV 
-# 
-# Parameters
-# x.pc - pre-clustered data values
-# Hstart - initial bandwidth matrices
+## Finds the bandwidth matrices that minimises LSCV 
+## 
+## Parameters
+## x.pc - pre-clustered data values
+## Hstart - initial bandwidth matrices
 #
-# Returns
-# H_PC,LSCV
+## Returns
+## H_PC,LSCV
 ###############################################################################
 
 Hlscv.pc <- function(x.pc, Hstart, univar=FALSE)
@@ -780,7 +793,7 @@ Hlscv.pc <- function(x.pc, Hstart, univar=FALSE)
   RK <- (4*pi)^(-d/2)
   ind.lab <- sort(unique(x.pc$ind))
   
-  # initial conditions based on normal reference b/w within each cluster
+  ## initial conditions based on normal reference b/w within each cluster
   if (missing(Hstart))
   {
     Hstart <- numeric()
@@ -797,7 +810,7 @@ Hlscv.pc <- function(x.pc, Hstart, univar=FALSE)
         #if (is.vector(xj))
           Hstart <- c(Hstart,sqrt(4/(n*(d + 2))^(2/(d + 4))))
         #else 
-        #  Hstart <- c(Hstart, sqrt((4/(n*(d + 2)))^(2/(d + 4))*max(abs(var(xj)))))
+        ##  Hstart <- c(Hstart, sqrt((4/(n*(d + 2)))^(2/(d + 4))*max(abs(var(xj)))))
     }                 
   }
 
@@ -810,7 +823,7 @@ Hlscv.pc <- function(x.pc, Hstart, univar=FALSE)
       H.pc <- numeric()
       for (j in 1:nu)
         H.pc  <- rbind(H.pc,  hpc[j]*hpc[j]*diag(d))
-        #  ensures that H.pc are positive definite
+        ##  ensures that H.pc are positive definite
        
       return(lscv.mat.pc(x.pc2, H.pc))
     }
@@ -837,7 +850,7 @@ Hlscv.pc <- function(x.pc, Hstart, univar=FALSE)
       {   
         vechHj <- vechHpc[((j-1)*1/2*dstar+1): (j*1/2*dstar)]
         H.pc  <- rbind(H.pc, invvech(vechHj) %*% invvech(vechHj))
-        #  ensures that H.pc are positive definite
+        ##  ensures that H.pc are positive definite
       }
       
       return(lscv.mat.pc(x.pc, H.pc))
